/* 
 *  Gwatch
 *  Copyright (C) 1999-2000 Peter Witkop
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>

#include <gtk/gtk.h>
#include <glib.h>

#include "mbox.h"
#include "gwatch.h"

int main (int argc, char *argv[])
{
  char *fname;

  gtk_init (&argc, &argv);

  fname = get_fname ();

#ifdef _DEBUG_
  printf ("spool name is: %s\n", fname);
#endif /* _DEBUG_ */
  gtk_timeout_add (_TIMEOUT_, timer_callback, fname);
  gtk_main ();

  return 0;
}

gint timer_callback (gpointer data)
{
  mbox mb;
  time_t mtime = 0;
  static time_t last_mtime = 0;
  struct stat fstatus;
  char *fname = (char *)data;

  if (stat (fname, &fstatus) != 0) {
    fprintf (stderr, "Warning: Cannon stat file %s\n", fname);
    return 1;
  }

#ifdef _USE_TIMESPEC_
  mtime = fstatus.st_mtimespec.tv_sec;
#else
  mtime = fstatus.st_mtime;
#endif /* _USE_TIMESPEC_ */

  if (mtime > last_mtime) {
    mbox_init (&mb, fname); 
    notify (&mb);
    mbox_free (&mb);
#ifdef _DEBUG_
    printf ("Timeout occured, spool changed: mtime = %i, last_mtime = %i\n",
	    mtime, last_mtime);
  }
  else {
    printf ("Timeout occured, spool unchanged: mtime = %i, last_mtime = %i\n",
	    mtime, last_mtime);     
#endif /* _DEBUG_ */
  }

  last_mtime = mtime;
  return 1;
}

void notify (mbox *mb)
{
  static GtkWidget *window = NULL;
  static GtkWidget *list;
  int last_new = 0;

  GtkWidget *vbox;
  GtkWidget *button;
  GtkWidget *dbox;
  GtkWidget *swindow;
  GtkWidget *sep;
  
  GList *m_list;

  static gchar *titles[2];
  static gchar lbl_from[] = {"From\0"};
  static gchar lbl_subject[] = {"Subject\0"};

  titles[0] = lbl_from;
  titles[1] = lbl_subject;

  m_list = mb->new_entries;
  last_new = get_last_new ();
  
  if (window != NULL) {
    if (last_new != mb->num_new)
      {
	gtk_clist_freeze (GTK_CLIST (list));
	gtk_clist_clear (GTK_CLIST (list));
	prop_clist (list, m_list);
	gtk_clist_thaw (GTK_CLIST (list));
      }

  }
  else {
    if (mb->num_new > last_new)
      {	
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL); 
	gtk_widget_realize (window);
	vbox = gtk_vbox_new (FALSE, 1);
	dbox = gtk_hbox_new (TRUE, 1);
	button = gtk_button_new_with_label (" dismis ");
	list = gtk_clist_new_with_titles (2, titles);
	swindow = gtk_scrolled_window_new (NULL, NULL);
	sep = gtk_hseparator_new ();
	
	gtk_window_set_default_size (GTK_WINDOW (window), 625, 150);
	gtk_container_set_border_width (GTK_CONTAINER (window), 1);
	gtk_window_set_title (GTK_WINDOW (window), "New Mail Has Arived");
	
	gtk_clist_set_selection_mode (GTK_CLIST (list), 
				      GTK_SELECTION_EXTENDED);
	gtk_clist_set_shadow_type (GTK_CLIST (list), 
				   GTK_SHADOW_IN);
	gtk_clist_column_titles_passive (GTK_CLIST (list));
	gtk_clist_set_column_width (GTK_CLIST (list), 0, 300);
	gtk_clist_set_column_width (GTK_CLIST (list), 1, 270);
	prop_clist (list, m_list);
	
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow),
					GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	
	gtk_signal_connect (GTK_OBJECT (button), "clicked", 
			    GTK_SIGNAL_FUNC (destroy_cb), &window);
	
	gtk_signal_connect (GTK_OBJECT (window), "destroy",
			    GTK_SIGNAL_FUNC (destroy_cb), &window);

	/*	
	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (swindow),
					       list);
	*/
	gtk_container_add (GTK_CONTAINER(swindow), list);
	gtk_widget_show (list);
	
	gtk_box_pack_start (GTK_BOX (vbox), swindow, TRUE, TRUE, 0);
	gtk_widget_show (swindow);
	
	gtk_box_pack_start (GTK_BOX (vbox), sep, FALSE, FALSE, 2);
	gtk_widget_show (sep);
	
	gtk_box_pack_end (GTK_BOX (dbox), button, FALSE, FALSE, 0);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (button);
	gtk_widget_show (button);
	
	gtk_box_pack_end (GTK_BOX (vbox), dbox, FALSE, FALSE, 0);
	gtk_widget_show (dbox);
	
	gtk_container_add (GTK_CONTAINER (window), vbox);
	gtk_widget_show (vbox);
	gtk_widget_show (window);
      }
  }
  set_last_new (mb->num_new);
}

void destroy_cb (GtkWidget *widget, gpointer data)
{
  GtkWidget *window = *(GtkWidget **)data;
  
  gtk_widget_hide (window);
  gtk_widget_destroy (window);

  /* if the data isn't null, notify will think the window
     is still there
  */
  *(GtkWidget **)data = NULL;
}

void prop_clist (GtkWidget *clist, GList *data)
{
  GList *lnode;
  mbox_entry *entry;
  char *row[2];
  int i;
  
  for (i = 0; i < g_list_length (data); i++)
    {
      lnode = g_list_nth (data, i);
      entry = (mbox_entry *)lnode->data;
      row[0] = entry->from;
      row[1] = entry->subject;
      gtk_clist_append (GTK_CLIST (clist), row);
    }
}

char *get_fname ()
{
  char *eval = getenv (_GWATCH_ENV_);
  char *rval;
  int len;

  if (eval != NULL)
    {
      len = strlen (eval);
      rval = (char *)malloc (len + 1);
      memset (rval, '\0', (len + 1));
      strncpy (rval, eval, len);
      
      return rval;
    }
  else if (getenv ("MAIL") != NULL)
    {
      eval = getenv ("MAIL");
      len = strlen (eval);
      rval = (char *)malloc (len + 1);
      memset (rval, '\0', (len + 1));
      strncpy (rval, eval, len);
      
      return rval; 
    }
  else 
    {
      eval = getenv ("USER");
      len = (strlen (_MAIL_DIR_) + strlen (eval));
      rval = malloc (len + 1);
      memset (rval, '\0', (len + 1));
      strcpy (rval, _MAIL_DIR_);
      strcat (rval, eval);
      
      return rval;
    }
}

int get_last_new ()
{
  char buff[15];
  char *rbuff;
  struct stat sbuff;
  int rval;
  FILE *file;
  int fname_len = strlen (getenv ("HOME")) + strlen (_CONF_DIR_) + 
                  strlen (_NEW_FILE_);
  char *mt_fname = (char *)malloc (fname_len + 1);
  memset (mt_fname, '\0', (fname_len + 1));
  strcat (mt_fname, getenv ("HOME"));
  strcat (mt_fname, "/");
  strcat (mt_fname, _CONF_DIR_);

#ifdef _DEBUG_
  printf ("new directory is: %s\n", mt_fname);
#endif /* _DEBUG_ */

  if (stat (mt_fname, &sbuff) == -1) {
    if (!make_conf_dir (mt_fname)) {
      fprintf (stderr, "Error creating configuration directory\n");
      return 0;
    }    
  }
  strcat (mt_fname, "/");
  strcat (mt_fname, _NEW_FILE_);

#ifdef _DEBUG_
  printf ("new file is: %s\n", mt_fname);
#endif /* _DEBUG_ */

  if (stat (mt_fname, &sbuff) == -1) {
    if (!make_file (mt_fname)) {
      fprintf (stderr, "Error creating message number sotrage file\n");
      return 0;
    }
    return 0;
  }

  file = fopen (mt_fname, "r+");
  if (!file) {
    fprintf (stderr, "Error opening new messages storage file\n");
    return 0;
  }

  rbuff = buff;
  memset (rbuff, '\0', 15);
  fgets (rbuff, 15, file);
  if (rbuff == NULL) {
    fprintf (stderr, "Warning, read 0 bytes from file\n");
    fclose (file);
    return 0;
  }
  fclose (file);
  rval = atoi (rbuff);

#ifdef _DEBUG_
  printf ("new is: %i, in string form %s\n", rval, rbuff);
#endif /* _DEBUG_ */

  free (mt_fname);
  return rval;
}

void set_last_new (int new)
{
  FILE *file;

  int fname_len = strlen (getenv ("HOME")) + strlen (_CONF_DIR_) + 
                  strlen (_NEW_FILE_);
  char *mt_fname = (char *)malloc (fname_len + 3);
  memset (mt_fname, '\0', (fname_len + 3));
  strcat (mt_fname, getenv ("HOME"));
  strcat (mt_fname, "/");
  strcat (mt_fname, _CONF_DIR_);  
  strcat (mt_fname, "/");
  strcat (mt_fname, _NEW_FILE_);

  file = fopen (mt_fname, "w");
  if (!file) {
    fprintf (stderr, "Error writing to modification time file\n");
    free (mt_fname);
    return;
  }

  free (mt_fname);
  fprintf (file, "%i\n", new);
  fclose (file);
}

int make_conf_dir (char *dname)
{
  if (mkdir (dname, (S_IRUSR | S_IWUSR | S_IXUSR)) == -1) {
    return -1;
  }  
  return 1;
}

int make_file (char *fname)
{
  int fd;

  fd = open (fname, (O_RDWR | O_CREAT));
  if (fd == -1) {
    fprintf (stderr, "Error creating file %s\n", fname);
    return fd;
  }
  close (fd);
  chmod (fname, (S_IRUSR | S_IWUSR));
  return 1;
}








