/*
 *  Gwatch
 *  Copyright (C) 1999-2000 Peter Witkop
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <gtk/gtk.h>

#include "mbox.h"
#include "gwatch.h"

int mbox_init (mbox *t_mbx, char *fname)
{
  char line[_MAX_LINE_LEN + 1];
  char cline;
  char *go_on, *temp_ptr;
  char *from = NULL;
  char *subject = NULL;
  FILE *inf;
  mbox_entry *mbx_entry;
  int new = 0, i, temp_size;
  int got_status = 0;
  int got_lines = 0;
  int loop_more = 1;
  fpos_t f_pos = 0;
  fpos_t cf_pos = 0;

  memset (line, '\0', (_MAX_LINE_LEN + 1));
  t_mbx->num_new = 0;
  t_mbx->new_entries = NULL;
  
  inf = fopen (fname, "r");

  if (inf == NULL) {
    fprintf (stderr, "Error opening file: %s\n", fname);
    return -1;
  }

  go_on = fgets (line, _MAX_LINE_LEN, inf);
  if (go_on == NULL) {
#ifdef _DEBUG_
    printf ("File opened, but we're not reading from it\n");
#endif /* _DEBUG_ */
    return -2;
  }

  while (go_on != NULL)
    {
      if (line[0] == '\n') /* Means we're at the end of a header */
	{
	  /* If there's no status line, we'll call it new */
	  if (got_status == 0) {
	    new = 1;
	    got_status = 0;
	  }

	  /* If it was a new message, add the info into the list */
	  if (new == 1)
	    {
	      mbx_entry = (mbox_entry *)malloc (sizeof (mbox_entry));
              t_mbx->num_new ++;
              if (from != NULL)
                { 
		  temp_size = strlen (from);
                  mbx_entry->from = (char *)malloc (temp_size + 1);
                  memset (mbx_entry->from, '\0', (temp_size + 1 ));
                  strncpy (mbx_entry->from, from, temp_size);
                }
              else 
                {
		  temp_size = strlen (_DEFAULT_FROM);
                  mbx_entry->from = (char *)malloc (temp_size + 1);
                  memset (mbx_entry->from, '\0', (temp_size + 1));
                  strncpy (mbx_entry->from, _DEFAULT_FROM, temp_size);
                }
              
              if (subject != NULL)
                {
		  temp_size = strlen (subject);
                  mbx_entry->subject = (char *)malloc (temp_size + 1);
                  memset (mbx_entry->subject, '\0', (temp_size + 1 ));
                  strncpy (mbx_entry->subject, subject, temp_size);
                }
              else 
                {
		  temp_size = strlen (_DEFAULT_SUBJ);
                  mbx_entry->subject = (char *)malloc (temp_size + 1);
                  memset (mbx_entry->subject, '\0', (temp_size + 1));
                  strncpy (mbx_entry->subject, _DEFAULT_SUBJ, temp_size);
                }
              t_mbx->new_entries = g_list_append (t_mbx->new_entries, 
                                                  (gpointer)mbx_entry);
              if (t_mbx->new_entries == NULL)
                fprintf (stderr, "Error adding message data to list\n");
	    }

	  if (from != NULL)
	    {
	      free (from);
	      from = NULL;
	    }
	  
	  if (subject != NULL)
	    {
	      free (subject);
	      subject = NULL;
	    }
	  
	  
	  /* Now move down to the next header */
	  if (got_lines > 0) /* If we found a "Lines" line */
	    {
	      for (i = 0; i <= got_lines; i++)
		{
		  go_on = fgets (line, _MAX_LINE_LEN, inf);
		}
	      got_lines = 0;
	    }
	  else /* If not, try to figure out were the next header starts */
	    {
	      while (loop_more)
		{
		  fgetpos (inf, &f_pos);
		  cline = line[0];
		  go_on = fgets (line, _MAX_LINE_LEN, inf);

		  if (go_on == NULL) {
		    loop_more = 0;
		  }
		  else {
		    if (strncmp (line, "From", 4) == 0) {
		      if (cline == '\n') {
			fgetpos (inf, &cf_pos);
			go_on = fgets (line, _MAX_LINE_LEN, inf);
			/** We're there! **/
			if (in_header (line)) {
			  loop_more = 0;
			  cline = '\0';
			}
			else { /** false alarm, we're not really there **/
			  fsetpos (inf, &cf_pos);
			  loop_more = 1;
			  cline = '\0';
			}
		      }
		      else {
			loop_more = 1;
		      }  
		    }
		  }
		}
	      
	      /* Move back up one line, so the from will be seen */
	      if (go_on != NULL)
		fsetpos (inf, &f_pos);

	      loop_more = 1;
	    }
	}
      else /* Looking for lines in the header */
	{
	  if (strncmp (line, "From", 4) == 0)
	    {
	      temp_size = strlen (line);
	      temp_ptr = line + 5;
	      if (*temp_ptr == ' ')
		temp_ptr ++;
	      if (from != NULL)
		free (from);
	      from = (char *)malloc (temp_size + 1);
	      memset (from, '\0', (temp_size + 1));
	      strncpy (from, temp_ptr, (temp_size - 5));
	      chop_nline (from);
	      temp_size = 0;
	      temp_ptr = NULL;
	    }
	  else if (strncmp (line, "Subject", 7) == 0)
	    {
	      temp_size = strlen (line);
	      temp_ptr = line + 9;
	      subject = (char *)malloc (temp_size + 1);
	      memset (subject, '\0', (temp_size + 1));
	      strncpy (subject, temp_ptr, (temp_size - 7));
	      chop_nline (subject);
	      temp_size = 0;
	    }
	  else if (strncmp (line, "Lines", 5) == 0)
	    {
	      temp_ptr = line + 7;
	      got_lines = atoi (temp_ptr);
	    }
	  else if (strncmp (line, "Status", 6) == 0)
	    {
	      got_status = 1;
	      if (strchr (line, 'R') == NULL)
		new = 1;
	    }
	}

      go_on = fgets (line, _MAX_LINE_LEN, inf);
    }

  fclose (inf);

  return 1;
}


void mbox_free (mbox *mbx)
{
  if (mbx != NULL) 
    {
      g_list_foreach (mbx->new_entries, free_func, NULL);  
      mbx->num_new = 0;
    }
}

void free_func (gpointer data, gpointer user_data)
{ mbox_entry_free ((mbox_entry *)data); }

void mbox_entry_free (mbox_entry *entry)
{
  if (entry != NULL) {
    if (entry->from != NULL) {
      free (entry->from);
    }
    if (entry->subject != NULL) {
      free (entry->subject);
    }
  }
}

void chop_nline (char *line)
{
  char *ltr = strrchr (line, '\n');
  if (ltr != NULL)
    *ltr = '\0'; 
}

int in_header (char *line)
{
  if ((strncmp (line, "Date", 4) == 0) ||
      (strncmp (line, "Return", 6) == 0)) {
    return 1;
  }
  else{
    return 0;
  }
}



